@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 our $VERSION;
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 =head1 NAME
 
@@ -91,12 +91,13 @@ sub send {
 	my $qtype   = $query->{qtype};
 	my $qclass  = $query->{qclass};
 
-	my $ctx	   = $self->{ub_ctx};
-	my $result = eval { Net::DNS::Resolver::libunbound::ub_resolve( $ctx, $qname, $qtype, $qclass ) };
+	my $ub_ctx = $self->{ub_ctx};
+	my $result = eval {
+		bless( Net::DNS::Resolver::libunbound::ub_resolve( $ub_ctx, $qname, $qtype, $qclass ),
+			'Net::DNS::Resolver::Unbound::Result' );
+	};
 	$self->errorstring($@);
-	my $packet = $self->_decode_result($result);
-	Net::DNS::Resolver::libunbound::ub_resolve_free($result);
-	return $packet;
+	return $self->_decode_result($result);
 }
 
 sub bgsend {
@@ -108,12 +109,10 @@ sub bgsend {
 	my $qtype   = $query->{qtype};
 	my $qclass  = $query->{qclass};
 
-	my $handle   = [];
-	my $async_id = eval {
-		Net::DNS::Resolver::libunbound::ub_resolve_async( $self->{ub_ctx}, $qname, $qtype, $qclass, $handle );
-	};
+	my $handle = [];
+	my $ub_ctx = $self->{ub_ctx};
+	eval { Net::DNS::Resolver::libunbound::ub_resolve_async( $ub_ctx, $qname, $qtype, $qclass, $handle ) };
 	$self->errorstring($@);
-	unshift( @$handle, $async_id );
 	return $handle;
 }
 
@@ -122,11 +121,10 @@ sub bgbusy {
 	return unless $handle;
 	my ( undef, @pre ) = @$handle;
 	return if scalar(@pre);
-	my $ctx = $self->{ub_ctx};
-	eval { Net::DNS::Resolver::libunbound::ub_process($ctx) }
-			if Net::DNS::Resolver::libunbound::ub_poll($ctx);
+	my $ub_ctx = $self->{ub_ctx};
+	eval { Net::DNS::Resolver::libunbound::ub_process($ub_ctx) };
 	$self->errorstring($@);
-	eval { select( undef, undef, undef, 0.200 ) };		# avoid tight loop on bgbusy()
+	eval { select( undef, undef, undef, 0.500 ) };		# avoid tight loop on bgbusy()
 	my ( undef, @post ) = @$handle;
 	return scalar(@post) ? 0 : 1;
 }
@@ -134,7 +132,8 @@ sub bgbusy {
 sub bgread {
 	my ( $self, $handle ) = @_;
 
-	eval { Net::DNS::Resolver::libunbound::ub_wait( $self->{ub_ctx} ) } if &bgbusy;
+	my $ub_ctx = $self->{ub_ctx};
+	eval { Net::DNS::Resolver::libunbound::ub_wait($ub_ctx) } if &bgbusy;
 	$self->errorstring($@);
 
 	return unless $handle;
